@@ -1,22 +1,23 @@
 /* srfi-60.c --- Integers as Bits
- *
- * Copyright (C) 2005, 2006, 2008, 2010, 2014 Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+
+   Copyright 2005-2006,2008,2010,2014,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -24,13 +25,16 @@
 # include <config.h>
 #endif
 
-#include "libguile/_scm.h"
-#include "libguile/eq.h"
+#include "boolean.h"
+#include "eq.h"
+#include "extensions.h"
+#include "gsubr.h"
+#include "list.h"
+#include "numbers.h"
+#include "pairs.h"
+#include "version.h"
 
-#include "libguile/validate.h"
-#include "libguile/numbers.h"
-
-#include "libguile/srfi-60.h"
+#include "srfi-60.h"
 
 
 SCM_DEFINE (scm_srfi60_log2_binary_factors, "log2-binary-factors", 1, 0, 0,
@@ -455,6 +459,6 @@ void
 scm_init_srfi_60 (void)
 {
 #ifndef SCM_MAGIC_SNARFER
-#include "libguile/srfi-60.x"
+#include "srfi-60.x"
 #endif
 }
